package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.crypto.digest.MD5;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.common.utils.DateTools;
import com.starbuds.server.api.notify.v1.entity.gamedev.*;
import com.starbuds.server.api.notify.v1.facade.GameDevFacade;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.message.im.game.LiveGameResultMsg;
import com.starbuds.server.common.service.message.ImService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.time.Duration;
import java.time.Instant;

/**
 * 游戏Dev接口实现
 * @author aoe
 * @date 2020/9/25
 */
@Path("/v1/gamedev")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class GameDevFacadeImpl extends BaseFacade implements GameDevFacade {

    @Reference(version = "1.0.0")
    private ImService imService;

    @Value("${gamedev.appId}")
    private String appId;

    @Value("${gamedev.apiSecretKey}")
    private String apiSecretKey;

    @Value("${gamedev.verifyMaxTimeDif}")
    private long verifyMaxTimeDif;

    /**
     * 接收游戏通知
     *
     * @param body
     * @return
     */
    @POST
    @Path("openapi")
    @PermitAll
    @Consumes("application/json")
    @Produces("application/json")
    @Override
    public String openApi(String body) {
        log.info("收到游戏Dev通知 {}", body);
        String verifyResult = verify(body);
        if (verifyResult != null) {
            log.warn(verifyResult);
            return verifyResult;
        }

        GameDevBaseNotify notify = JSONObject.parseObject(body, GameDevBaseNotify.class);
        String method = notify.getMethod();
        if ("winno_notify".equals(method)) {
            // 开奖通知
            receiveWinNoNotify(body);
        } else {
            // 中奖通知
            receiveWinNotify(body);
        }

        return "success";
    }

    /**
     * 验证数据
     * @param body
     * @return null:格式正确；错误信息
     */
    private String verify(String body) {
        if (StringUtils.isBlank(body)) {
            return "请求数据不能为空";
        }
        if (!JSON.isValidObject(body)) {
            return "请求数据必须是JSON格式";
        }
        GameDevBaseNotify notify = JSONObject.parseObject(body, GameDevBaseNotify.class);
        String method = notify.getMethod();
        if (!"winno_notify".equals(method) && !"win_notify".equals(method)) {
            return "无效的通知类型";
        }
        if (!appId.equals(notify.getAppid())) {
            return "appId不匹配";
        }
        Long timestamp = notify.getTimestamp();
        if (timestamp == null) {
            return "timestamp不能为空";
        }
        String sign = notify.getSign();
        if (StringUtils.isBlank(sign)) {
            return "签名不能为空";
        }
        if (!sign.equals(sign(timestamp, notify.getMethod()))) {
            return "无效的签名";
        }
        // 验证发送时间
        Instant receiveTime = Instant.ofEpochMilli(timestamp);
        Instant now = DateTools.nowInstant();
        long dif = Duration.between(receiveTime, now).toMillis();
        if (dif > verifyMaxTimeDif) {
            return "超过验证时间最大时差";
        }
        return null;
    }

    /**
     * 收到开奖通知
     * @param body
     */
    private void receiveWinNoNotify(String body) {
        WinNoNotify notify = JSONObject.parseObject(body, WinNoNotify.class);
        WinNoData data = notify.getData();
        if (data == null) {
            log.warn("收到开奖通知数据异常，data=null notify={}", notify);
            return;
        }
        WinNoMsg winNoMsg = new WinNoMsg();
        BeanUtils.copyProperties(data, winNoMsg);
        LiveGameResultMsg msg = new LiveGameResultMsg();
        msg.setNotifyType(10);
        msg.setResult(JSON.toJSONString(winNoMsg));
        log.info("游戏Dev发送IM 开奖通知 {}", msg);
        imService.sendLiveRoomMsg(ImTargetType.ALL, null, null, msg);
    }

    /**
     * 收到中奖通知
     * @param body
     */
    private void receiveWinNotify(String body) {
        WinNotify notify = JSONObject.parseObject(body, WinNotify.class);
        WinData data = notify.getData();
        if (data == null) {
            log.warn("收到中奖通知数据异常，data=null notify={}", notify);
            return;
        }
        WinMsg winMsg = new WinMsg();
        BeanUtils.copyProperties(data, winMsg);
        LiveGameResultMsg msg = new LiveGameResultMsg();
        msg.setNotifyType(20);
        msg.setResult(JSON.toJSONString(winMsg));
        log.info("游戏Dev发送IM 中奖通知 {}", msg);
        imService.sendLiveRoomMsg(ImTargetType.ALL, null, null, msg);
    }

    private String sign(long timestamp, String method) {
        // 签名规则：md5(appid+timestamp+method+secret)  --默认小写
        String signParam = appId + timestamp + method + apiSecretKey;
        return MD5.create().digestHex(signParam).toLowerCase();
    }

    public static void main(String[] args) {
        WinNoNotify notify = new WinNoNotify();
        notify.setAppid("10001001");
        notify.setMethod("winno_notify");
        notify.setTimestamp(System.currentTimeMillis());
        notify.setSign("签名");

        WinNoData data = new WinNoData();
        data.setGroupCode("游戏组");
        data.setGameId("游戏id");
        data.setGameName("游戏名称");
        data.setPeriod("开奖期号");
        data.setWinNo("开奖号码");
        notify.setData(data);

        System.out.println(JSON.toJSONString(notify));
    }
}
