package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.crypto.digest.MD5;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.common.utils.DateTools;
import com.starbuds.server.api.notify.v1.entity.GameOrderStatusNotify;
import com.starbuds.server.api.notify.v1.facade.GameFacade;
import com.starbuds.server.common.service.finance.GameService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.*;
import java.time.Duration;
import java.time.Instant;
import java.util.UUID;

/**
 * 游戏接口实现
 * @author aoe
 * @date 2020/9/15
 */
@Path("/v1/game")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class GameFacadeImpl extends BaseFacade implements GameFacade {

    @Reference(version = "1.0.0")
    private GameService gameService;

    @Value("${game.appId}")
    private String appId;

    @Value("${game.apiSecretKey}")
    private String apiSecretKey;

    @Value("${game.verifyMaxTimeDif}")
    private long verifyMaxTimeDif;

    /**
     * 接收订单处理结果
     * @param notify
     * @return
     */
    @POST
    @Path("receiveOrderStatus")
    @PermitAll
    @Override
    public JSONObject receiveOrderStatus(@BeanParam GameOrderStatusNotify notify) {
        log.info("收到游戏订单处理结果通知 {}", notify);
        String orderNo = notify.getOrderNo();

        if (!appId.equals(notify.getAppid())) {
            // appId错误
            return result(10, orderNo);
        }

        // 非空验证
        if (StringUtils.isBlank(notify.getTs())) {
            return result(20, orderNo);
        }
        if (StringUtils.isBlank(notify.getOrderNo())) {
            return result(21, orderNo);
        }
        if (StringUtils.isBlank(notify.getResult())) {
            return result(22, orderNo);
        }
        String sign = notify.getSign();
        if (StringUtils.isBlank(sign)) {
            // 签名为空
            return result(23, orderNo);
        }

        // 格式验证
        if (!NumberUtil.isLong(notify.getOrderNo())) {
            return result(30, orderNo);
        }
        if (!sign.toUpperCase().equals(sign(notify.getTs()))) {
            // 签名错误
            return result(40, orderNo);
        }
        // 验证发送时间
        Instant receiveTime = DateTools.toInstant(notify.getTs(), "yyyy-MM-dd HH:mm:ss");
        Instant now = DateTools.nowInstant();
        long dif = Duration.between(receiveTime, now).toMillis();
        if (dif > verifyMaxTimeDif) {
            // 超过验证时间最大时差
            return result(41, orderNo);
        }

        // result 转账结果：1初始，2成功，3失败
        boolean flag = gameService.updateGameCoinOrder(NumberUtil.parseLong(notify.getOrderNo()),
                "2".equals(notify.getResult()), "第三方返回result=" + notify.getResult());

        // 1成功，0失败
        return result(flag ? 1 : 0, orderNo);
    }

    /**
     *
     * @param code 1成功，0失败
     * @param orderNo 转账订单号
     * @return
     */
    private JSONObject result(int code, String orderNo) {
        if (code != 1) {
            log.warn("收到游戏订单处理结果通知，参数验证失败 code={} orderNo={}", code, orderNo);
        }
        JSONObject json = new JSONObject();
        json.put("code", code);
        json.put("request_id", UUID.randomUUID().toString());
        return json;
    }

    private String sign(String ts) {
        // 签名规则：md5(appid+ts+secret)
        String signParam = appId + ts + apiSecretKey;
        return MD5.create().digestHex(signParam).toUpperCase();
    }

    public static void main(String[] args) {
        String ts = "2020-09-22 14:48:15";
        String appId = "8da954c1-e43e-4c00-b7f7-882e8caafe5b";
        String apiSecretKey = "5F13F6A4DE1B49F724036AA7";
        String signParam = appId + ts + apiSecretKey;
        // 240D49A2AF2430F31C0E9B4B99C56A80
        System.out.println(MD5.create().digestHex(signParam));
        // GameOrderStatusNotify(appid=8da954c1-e43e-4c00-b7f7-882e8caafe5b, platform=BBIN, playerName=tes18077106307101, ts=2020-09-22 14:48:15, sign=240D49A2AF2430F31C0E9B4B99C56A80, tranType=2, amount=1.0, orderNo=214980670521346, result=2)

    }

/*
curl --header "Content-Type: application/json" \
  --request POST \
  --data '{"appid":"8da954c1-e43e-4c00-b7f7-882e8caafe5b","platform":"BBIN","player_name":"test20200723","player_password":"123456","ts":"2020-07-23 05:01:30","sign":"5f17bab62d197730ee672d30","tran_type":"1","amount":"1","order_no":"1600334468","result":"1"}' \
  http://api.timichat.net/api-notify/v1/game/receiveOrderStatus -t utf-8
*/
}
