package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import com.starbuds.server.api.notify.v1.entity.TencentNotifyResult;
import com.starbuds.server.api.notify.v1.facade.ImFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.service.message.ImService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.regex.Pattern;

/**
 * IM 消息通知实现
 *
 * @author Lay
 * @date 2019/12/24
 */
@Path("/v1/im")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ImFacadeImpl extends BaseFacade implements ImFacade {

    @Reference(version = "1.0.0")
    ImService imService;

    private static final Pattern pattern = Pattern.compile("\\\\n\\s+");

    @POST
    @Path("tencent")
    @PermitAll
    @Override
    public TencentNotifyResult tencent(@QueryParam("SdkAppid") Long sdkAppId,
                                       @QueryParam("CallbackCommand") String callbackCommand,
                                       @QueryParam("ClientIP") String clientIp,
                                       @QueryParam("OptPlatform") String optPlatform,
                                       @Context HttpServletRequest request) {
        // 判断 SdkAppId
        if (!imService.checkSdkAppId(sdkAppId)) {
            return buildError("SdkAppId 不匹配");
        }

        // 处理
        try {
            String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
            log.info("收到腾讯IM通知 {}", body);
            if (imService.handleTencentImCallback(callbackCommand, clientIp, optPlatform, body)) {
                return buildSuccess();
            }
        } catch (Exception e) {
            log.error("腾讯 IM 回调处理失败", e);
        }

        return buildError("");
    }

    @POST
    @Path("roomStatus")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String roomStatus(@QueryParam("signTimestamp") String signTimestamp,
                             @QueryParam("nonce") String nonce,
                             @QueryParam("signature") String signature,
                             @Context HttpServletRequest request) {
        if (StringUtils.isNotBlank(signTimestamp) && StringUtils.isNotBlank(nonce) && StringUtils.isNotBlank(signature)) {
            // 验签
            if (imService.checkRongCloudSign(signTimestamp, nonce, signature)) {
                // 处理
                try {
                    String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
                    log.info("收到融云通知 {}", body);
                    imService.handleRongCloudRoomStatusNotify(body);
                } catch (Exception e) {
                    log.error("融云房间状态同步通知处理失败", e);
                }
            }
        }

        return null;
    }

    @POST
    @Path("onlineStatus")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String onlineStatus(@QueryParam("signTimestamp") String signTimestamp,
                               @QueryParam("nonce") String nonce,
                               @QueryParam("signature") String signature,
                               @Context HttpServletRequest request) {
        if (StringUtils.isNotBlank(signTimestamp) && StringUtils.isNotBlank(nonce) && StringUtils.isNotBlank(signature)) {
            // 验签
            if (imService.checkRongCloudSign(signTimestamp, nonce, signature)) {
                // 处理
                try {
                    String body = ReUtil.replaceAll(IOUtils.toString(request.getInputStream(), CharsetUtil.UTF_8), pattern, "");
                    log.info("收到融云通知 {}", body);
                    imService.handleRongCloudUserStatusNotify(body);
                } catch (Exception e) {
                    log.error("融云用户在线状态同步通知处理失败", e);
                }
            }
        }

        return "success";
    }

    private static TencentNotifyResult buildError(String reason) {
        return TencentNotifyResult.builder().errorCode(BoolIntVal.True).actionStatus("FAIL").errorInfo(reason).build();
    }

    private static TencentNotifyResult buildSuccess() {
        return TencentNotifyResult.builder().errorCode(BoolIntVal.False).actionStatus("OK").errorInfo("").build();
    }
}
