package com.starbuds.server.api.notify.v1.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.notify.constant.AlipayTradeStatus;
import com.starbuds.server.api.notify.constant.HuaweiNotifyResultEnum;
import com.starbuds.server.api.notify.util.RequestUtil;
import com.starbuds.server.api.notify.v1.entity.AlipayNotify;
import com.starbuds.server.api.notify.v1.entity.HuaweiNotify;
import com.starbuds.server.api.notify.v1.entity.HuaweiNotifyResult;
import com.starbuds.server.api.notify.v1.entity.WxpayNotifyResult;
import com.starbuds.server.api.notify.v1.facade.PayFacade;
import com.starbuds.server.common.pojo.finance.PayNotify;
import com.starbuds.server.common.service.finance.PayService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.jboss.resteasy.annotations.Form;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 支付通知接口实现
 *
 * @author Lay
 * @date 2019/12/20
 */
@Path("/v1/pay")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class PayFacadeImpl extends BaseFacade implements PayFacade {
    private static final String aliFail = "fail";
    private static final String aliSuccess = "success";

    @Reference(version = "1.0.0")
    PayService payService;

    @POST
    @Path("alipay")
    @PermitAll
    @Override
    public String alipay(@Form AlipayNotify notify, @Context HttpServletRequest request) {
        log.info("支付宝异步通知 {}", JSON.toJSONString(notify));

        // 验签
        Map<String, String> mapParams = new HashMap<>();
        for (String key : request.getParameterMap().keySet()) {
            mapParams.put(key, request.getParameter(key));
        }

        if (!payService.checkAlipayNotifySign(Convert.toLong(notify.getPassbackParams()), notify.getAppId(), notify.getCharset(), notify.getSignType(), mapParams)) {
            log.warn("验签失败");
            return aliFail;
        }

        // 处理订单逻辑
        if (notify.getTradeStatus().equals(AlipayTradeStatus.TRADE_SUCCESS)) {
            // 交易成功
            PayNotify payNotify = new PayNotify();
            payNotify.setPayId(Convert.toLong(notify.getOutTradeNo()));
            payNotify.setBuyerId(notify.getBuyerId());
            payNotify.setBuyerAccount(notify.getBuyerLogonId());
            payNotify.setSellerId(notify.getSellerId());
            payNotify.setSellerAccount(notify.getSellerId());
            payNotify.setTotalAmount(notify.getTotalAmount());
            payNotify.setNotifyId(notify.getNotifyId());

            if (!payService.handlePayNotify(payNotify)) {
                return aliFail;
            }

        } else if (notify.getTradeStatus().equals(AlipayTradeStatus.TRADE_CLOSED)) {
            // 交易关闭
        }

        return aliSuccess;
    }

    @POST
    @Path("wxpay")
    @Produces(ContentType.TEXT_XML_UTF_8)
    @PermitAll
    @Override
    public WxpayNotifyResult wxpay(@Context HttpServletRequest request) {
        try {
            String xmlData = RequestUtil.httpServletRequestToString(request);
            log.info("微信支付异步通知: {}", xmlData);
            // 验签
            if (payService.checkWxpayNotifySign(xmlData)) {
                // 处理订单逻辑
                if (!payService.handleWxpayNotify(xmlData)) {
                    getNotifyResult("订单处理失败", false);
                }

                return getNotifyResult("订单处理成功", true);
            }
            return getNotifyResult("签名校验失败", false);
        } catch (Exception e) {
            return getNotifyResult("参数格式校验错误", false);
        }
    }

    @POST
    @Path("huawei")
    @PermitAll
    @Override
    public HuaweiNotifyResult huawei(@Form HuaweiNotify notify, @Context HttpServletRequest request) {
        try {
            log.info("华为异步通知 {}", JSON.toJSONString(notify));

            // 验签
            Map<String, Object> mapParams = new HashMap<>();
            for (String key : request.getParameterMap().keySet()) {
                mapParams.put(key, request.getParameter(key));
            }

            if (!payService.checkHuaweiNotifySign(Convert.toLong(notify.getExtReserved()), mapParams)) {
                log.warn("验签失败");
                return HuaweiNotifyResult.build(HuaweiNotifyResultEnum.SIGN_CHECK_FAIL);
            }

            // 处理订单逻辑
            if (notify.getResult().equals("0")) {
                // 交易成功
                PayNotify payNotify = new PayNotify();
                payNotify.setPayId(Convert.toLong(notify.getRequestId()));
                payNotify.setSellerId(notify.getUserName());
                payNotify.setTotalAmount(new BigDecimal(notify.getAmount()));
                payNotify.setNotifyId(notify.getOrderId());

                if (payService.handlePayNotify(payNotify)) {
                    return HuaweiNotifyResult.build(HuaweiNotifyResultEnum.SUCCESS);
                }
            }
            return HuaweiNotifyResult.build(HuaweiNotifyResultEnum.BIZ_ERR);
        } catch (Exception e) {
            return HuaweiNotifyResult.build(HuaweiNotifyResultEnum.SYS_ERR);
        }
    }

    private WxpayNotifyResult getNotifyResult(String message, boolean isSuccess) {
        log.info(message);
        WxpayNotifyResult notifyResult = new WxpayNotifyResult();
        notifyResult.setReturn_code(isSuccess ? "SUCCESS" : "FAIL");
        notifyResult.setReturn_msg(message);
        return notifyResult;
    }
}
